import {ItemStack, BlockPermutation} from "@minecraft/server";
import * as inventory                from "../utility/inventory.js";

const permutate = {
	onPlayerInteract({player, block, face}) {interact(player, block, face)},
	onPlace({block}) {update_blocks(block)},
	onPlayerDestroy({block}) {update_blocks(block)}
};

function interact(player, block, face) {
	if (player.isSneaking) return;
	if (face.toLowerCase() !== block.permutation.getState("minecraft:cardinal_direction")) return;

	if (block.typeId === "tms_mlb:display.jersey.absent") {
		absent(player, block);
	} else {
		present(player, block);
	}
}

function absent(player, block) {
	const item = player.getComponent("equippable")?.getEquipment("Mainhand");
	const item_id = item?.typeId || "";
	const [item_prefix, item_suffix] = item_id.split(".");

	if (item_prefix !== "tms_mlb:jersey") return;
	if (item_suffix === "absent") return;

	const new_states = BlockPermutation.resolve(
		item_id.replace("jersey", "display.jersey"),
		block.permutation.getAllStates()
	);
	block.setPermutation(new_states);

	player.playSound("random.pop", {
		pitch : Math.round((Math.random() * 0.25 + 0.5) * 100) / 100,
		volume: 0.3
	});

	if (player.getGameMode() === "creative") return;

	inventory.consume(player, item);
}

function present(player, block) {
	const new_item  = new ItemStack(block.typeId.replace("display.jersey", "jersey"), 1);
	inventory.give(player, new_item);

	const new_states = BlockPermutation.resolve( // COMMENT: This statement MUST be after new_item declaration, otherwise the player get a blank jersey
		"tms_mlb:display.jersey.absent",
		block.permutation.getAllStates()
	);
	block.setPermutation(new_states);
}

function update_blocks(block) {

	const blocks = [
		block,
		block.above(1),
		block.above(1).north(1),
		block.above(1).south(1),
		block.above(1).east(1),
		block.above(1).west(1),
		block.below(1),
		block.below(1).north(1),
		block.below(1).south(1),
		block.below(1).east(1),
		block.below(1).west(1),
		block.north(1),
		block.south(1),
		block.east(1),
		block.west(1)
	];
	blocks.forEach(block => {
		if (!block.typeId.startsWith("tms_mlb:display.jersey")) return;

		const direction = block.permutation.getState("minecraft:cardinal_direction");
		
		const blocks = [
			block.above(1),
			block.below(1)
		];
		switch (direction) {
			case "north": blocks.push (block.west(1) , block.east(1) , block.west(1).above(1) , block.east(1).above(1) , block.west(1).below(1) , block.east(1).below(1)) ; break;
			case "south": blocks.push (block.east(1) , block.west(1) , block.east(1).above(1) , block.west(1).above(1) , block.east(1).below(1) , block.west(1).below(1)) ; break;
			case "east" : blocks.push (block.north(1), block.south(1), block.north(1).above(1), block.south(1).above(1), block.north(1).below(1), block.south(1).below(1)); break;
			case "west" : blocks.push (block.south(1), block.north(1), block.south(1).above(1), block.north(1).above(1), block.south(1).below(1), block.north(1).below(1)); break;
			default: break;
		}

		const matches = block => block.typeId.startsWith("tms_mlb:display.jersey") && block.permutation.getState("minecraft:cardinal_direction") === direction;

		const new_states = BlockPermutation.resolve(block.typeId, {
			"tms_mlb:top"         : matches(blocks[0]),
			"tms_mlb:bottom"      : matches(blocks[1]),
			"tms_mlb:left"        : matches(blocks[2]),
			"tms_mlb:right"       : matches(blocks[3]),
			"tms_mlb:top_left"    : matches(blocks[0]) || matches(blocks[2]) || matches(blocks[4]),
			"tms_mlb:top_right"   : matches(blocks[0]) || matches(blocks[3]) || matches(blocks[5]),
			"tms_mlb:bottom_left" : matches(blocks[1]) || matches(blocks[2]) || matches(blocks[6]),
			"tms_mlb:bottom_right": matches(blocks[1]) || matches(blocks[3]) || matches(blocks[7]),
			"minecraft:cardinal_direction": block.permutation.getState("minecraft:cardinal_direction")
		});
		block.setPermutation(new_states);
	});
}

export {
	permutate
};