import {ItemStack, BlockPermutation} from "@minecraft/server";
import * as inventory                from "../utility/inventory.js";

const interact = {onPlayerInteract({block, player}) {

	if (player.isSneaking) return;

	const item      = player.getComponent("equippable").getEquipment("Mainhand");
	const item_id   = item?.typeId || "";
	const amount    = block.permutation.getState("tms_mlb:amount");

	if (item_id === "minecraft:wheat_seeds" && amount < 4) pop_popcorn(block, amount, player, item);
	if (item_id !== "minecraft:wheat_seeds" && amount > 0) get_popcorn(block, amount, player);
}};

function pop_popcorn(block, amount, player, item) {
	const new_states = BlockPermutation.resolve(
		"tms_mlb:popcorn_machine",
		{"tms_mlb:amount": amount + 1}
	);
	block.setPermutation(new_states);

	const location = {x: block.x + 0.5, y: block.y, z: block.z + 0.5};
	
	block.dimension.playSound(    "tms_mlb:popcorn", location);
	block.dimension.spawnParticle("tms_mlb:popcorn", location);

	if (player.getGameMode() === "creative") return;

	inventory.consume(player, item);
}

function get_popcorn(block, amount, player) {
	const new_states = BlockPermutation.resolve(
		"tms_mlb:popcorn_machine",
		{"tms_mlb:amount": amount - 1}
	);
	block.setPermutation(new_states);

	player.playSound("random.pop", {
		pitch : Math.round((Math.random() * 0.75 + 1) * 100) / 100,
		volume: 0.3
	});

	const new_item = new ItemStack("tms_mlb:popcorn", 1);
	inventory.give(player, new_item);
}

export {
	interact
};