import {BlockPermutation} from "@minecraft/server";

const seat_map = {
	"tms_mlb:seat.glove"           : "tms_mlb:seat.3.locked",
	"tms_mlb:seat.beanbag.baseball": "tms_mlb:seat.3",
	"tms_mlb:seat.stool.baseball"  : "tms_mlb:seat.8",
	"tms_mlb:seat.folding"         : "tms_mlb:seat.3.locked",
	"tms_mlb:seat.stadium"         : "tms_mlb:seat.3.locked",
	"tms_mlb:seat.dining"          : "tms_mlb:seat.3.locked",
	"tms_mlb:seat.couch"           : "tms_mlb:seat.3.locked"
};

const direction_to_rotation = {
	north: 180,
	south:   0,
	east : -90,
	west :  90
};

const interact = {onPlayerInteract({block, face, player}) {
	if (player.isSneaking) return;

	if (block.typeId === "tms_mlb:seat.stadium") {
		const stadium_state = block.permutation.getState("tms_mlb:open");
		if (!stadium_state && face !== "Up" && face !== "Down") stadium_open(block);
		if ( stadium_state && face !== "Up" && face !== "Down") stadium_close(block);
	}

	if (face !== "Up") return;
	if (block.dimension.getEntities({
		families   : ["tms_mlb:seat"],
		location   : block.location,
		maxDistance: 1,
		closest    : 1
	}).length !== 0) return;

	create(block, player);
}};

function create(block, player) {
	const seat     = seat_map[block.typeId];
	const x        = block.x + 0.5;
	const y        = block.y;
	const z        = block.z + 0.5;
	const rotation = direction_to_rotation[block.permutation.getState("minecraft:cardinal_direction")];

	player.runCommand(`summon ${seat} ${x} ${y} ${z} ${rotation} 0`); // NOTE: Turning the runCommands into scripts might not be possible yet
	player.runCommand(`ride "${player.name}" start_riding @e[type=${seat},x=${x},y=${y},z=${z},c=1] teleport_rider`);
}

function stadium_enter (entity) {
	const block = entity.dimension.getBlock(entity.location);
	stadium_open(block);
}

function stadium_exit (entity) {
	const block = entity.dimension.getBlock(entity.location);
	stadium_close(block);
}

function stadium_close (block) { // TODO: Add seat despawn for when a player is riding and another player closes the seat
	block.dimension.playSound("close.iron_trapdoor", block.location);
	const new_states = BlockPermutation.resolve("tms_mlb:seat.stadium", {
		"tms_mlb:open": false,
		"minecraft:cardinal_direction": block.permutation.getState("minecraft:cardinal_direction")
	});
	block.setPermutation(new_states);
}

function stadium_open (block) {
	block.dimension.playSound("open.iron_trapdoor", block.location);
	const new_states = BlockPermutation.resolve("tms_mlb:seat.stadium", {
		"tms_mlb:open": true,
		"minecraft:cardinal_direction": block.permutation.getState("minecraft:cardinal_direction")
	});
	block.setPermutation(new_states);
}

const couch = {
	onPlace({block}) {update_blocks(block)},
	onPlayerDestroy({block}) {update_blocks(block)}
};

function update_blocks(block) {
	const blocks = [
		block,
		block.north(1),
		block.south(1),
		block.east(1),
		block.west(1)
	];
	blocks.forEach(block => {
		if (block.typeId !== "tms_mlb:seat.couch") return;

		const direction = block.permutation.getState("minecraft:cardinal_direction");

		const blocks = [];
		switch (direction) {
			case "north": blocks.push (block.west(1) , block.east(1) ); break;
			case "south": blocks.push (block.east(1) , block.west(1) ); break;
			case "east" : blocks.push (block.north(1), block.south(1)); break;
			case "west" : blocks.push (block.south(1), block.north(1)); break;
			default: break;
		}
	
		const new_states = BlockPermutation.resolve("tms_mlb:seat.couch", {
			"tms_mlb:left" : blocks[0].typeId === "tms_mlb:seat.couch",
			"tms_mlb:right": blocks[1].typeId === "tms_mlb:seat.couch",
			"minecraft:cardinal_direction": direction
		});
	
		block.setPermutation(new_states);
	});
}

export {
	interact,
	stadium_enter,
	stadium_exit,
	couch
};