import {ItemStack} from "@minecraft/server";

function give(player, item) {
	const inventory = player.getComponent("minecraft:inventory").container;
	let index;
	let slot;
	for (let i = 0; i < 36; i++) {
		slot = inventory.getItem(i);
		if (!slot) continue;
		if (slot.typeId !== item.typeId) continue;
		if (slot.amount === item.maxAmount) continue;
		index = i;
		break;
	}
	if (!index && inventory.emptySlotsCount === 0) drop(player, item);
	if (!index && inventory.emptySlotsCount >   0) add(player, item, inventory);
	if (index) set(player, slot, inventory, index);
}

function drop(player, item) {
	const location = {
		x: player.location.x,
		y: player.location.y + 1.1,
		z: player.location.z
	};
	const item_entity = player.dimension.spawnItem(item, location);
	const view_direction = {
		x: player.getViewDirection().x * 0.3,
		y: player.getViewDirection().y * 0.3,
		z: player.getViewDirection().z * 0.3
	};
	item_entity.clearVelocity();
	item_entity.applyImpulse(view_direction);
	player.playSound("random.pop", {
		pitch : Math.round((Math.random() * 0.25 + 0.5) * 100) / 100,
		volume: 0.3
	});
}

function add(player, item, inventory) {
	inventory.addItem(item);
	player.playSound("random.pop", {
		pitch : Math.round((Math.random() * 0.75 + 1) * 100) / 100,
		volume: 0.3
	});
}

function set(player, item, inventory, index) {
	const new_item = new ItemStack(item.typeId, item.amount + 1);
	inventory.setItem(index, new_item);
	player.playSound("random.pop", {
		pitch : Math.round((Math.random() * 0.75 + 1) * 100) / 100,
		volume: 0.3
	});
}

function consume(player, item) {
	const inventory = player.getComponent("minecraft:inventory").container;
	const index = player.selectedSlotIndex;
	const new_amount = item.amount - 1;

	if (new_amount === 0) {
		inventory.setItem(index, undefined);
		return;
	}

	const new_item = new ItemStack(item.typeId, new_amount);
	inventory.setItem(index, new_item);
}

export {
	give,
	consume
};